import { pipeline } from 'stream';
import { promisify } from 'util';
import { bucketName, minioClient, minioResources } from '../minio.ts';
import fs from 'fs';
import { IncomingMessage, ServerResponse } from 'http';
import http from 'http';
import https from 'https';
import { UserApp } from '../get-user-app.ts';

const pipelineAsync = promisify(pipeline);

export async function downloadFileFromMinio(fileUrl: string, destFile: string) {
  const objectName = fileUrl.replace(minioResources + '/', '');
  const objectStream = await minioClient.getObject(bucketName, objectName);
  const destStream = fs.createWriteStream(destFile);
  await pipelineAsync(objectStream, destStream);
  console.log(`minio File downloaded to ${minioResources}/${objectName} \n ${destFile}`);
}
export async function minioProxy(
  req: IncomingMessage,
  res: ServerResponse,
  opts: {
    proxyUrl: string;
    userApp: UserApp;
    createNotFoundPage: (msg?: string) => any;
  },
) {
  const fileUrl = opts.proxyUrl;
  const { userApp, createNotFoundPage } = opts;
  const objectName = fileUrl.replace(minioResources + '/', '');
  try {
    const stat = await minioClient.statObject(bucketName, objectName);
    if (stat.size === 0) {
      return createNotFoundPage('Invalid proxy url');
    }
    const contentLength = stat.size;
    const etag = stat.etag;
    const lastModified = stat.lastModified.toISOString();
    // console.log('contentType', stat.metaData);
    res.writeHead(200, {
      'Content-Length': contentLength,
      etag,
      'last-modified': lastModified,
      ...stat.metaData,
    });
    const objectStream = await minioClient.getObject(bucketName, objectName);
    objectStream.pipe(res, { end: true });
  } catch (error) {
    console.error(`Proxy request error: ${error.message}`);
    userApp.clearCacheData();
    return createNotFoundPage('Invalid proxy url');
  }
}

export const httpProxy = async (
  req: IncomingMessage,
  res: ServerResponse,
  opts: {
    proxyUrl: string;
    userApp: UserApp;
    createNotFoundPage: (msg?: string) => any;
  },
) => {
  const { proxyUrl, userApp, createNotFoundPage } = opts;
  if (proxyUrl.startsWith(minioResources)) {
    return minioProxy(req, res, opts);
  }
  let protocol = proxyUrl.startsWith('https') ? https : http;
  // 代理
  const proxyReq = protocol.request(proxyUrl, (proxyRes) => {
    res.writeHead(proxyRes.statusCode, {
      ...proxyRes.headers,
    });
    if (proxyRes.statusCode === 404) {
      userApp.clearCacheData();
      return createNotFoundPage('Invalid proxy url');
    }
    if (proxyRes.statusCode === 302) {
      res.writeHead(302, { Location: proxyRes.headers.location });
      return res.end();
    }
    proxyRes.pipe(res, { end: true });
  });
  proxyReq.on('error', (err) => {
    console.error(`Proxy request error: ${err.message}`);
    userApp.clearCacheData();
  });
  proxyReq.end();
};
